@@ -20,10 +20,15 @@ from .document_template import DocumentTemplate
 from .extraction_mode import ExtractionMode
 from .generate_schema_body import GenerateSchemaBody
 from .generative_task_request import GenerativeTaskRequest
+from .knowledge_bases_controller_promote_knowledge_base_request import (
+    KnowledgeBasesControllerPromoteKnowledgeBaseRequest,
+)
 from .llm_key_binding_dto import LlmKeyBindingDto
 from .llm_provider_dto import LlmProviderDto
 from .llm_type import LlmType
 from .map_reduce_command import MapReduceCommand
+from .map_reduce_header_command import MapReduceHeaderCommand
+from .map_reduce_runtime_status import MapReduceRuntimeStatus
 from .named_configuration import NamedConfiguration
 from .ocr_command import OcrCommand
 from .ocr_document_sync_body import OcrDocumentSyncBody
@@ -91,10 +96,13 @@ __all__ = (
     "ExtractionMode",
     "GenerateSchemaBody",
     "GenerativeTaskRequest",
+    "KnowledgeBasesControllerPromoteKnowledgeBaseRequest",
     "LlmKeyBindingDto",
     "LlmProviderDto",
     "LlmType",
     "MapReduceCommand",
+    "MapReduceHeaderCommand",
+    "MapReduceRuntimeStatus",
     "NamedConfiguration",
     "OcrCommand",
     "OcrDocumentSyncBody",
