@@ -1,10 +1,11 @@
 from typing import Literal, cast
 
-OcrType = Literal["AUTO", "DEFAULT", "LOW", "NONE", "PREMIUM"]
+OcrType = Literal["AUTO", "DEFAULT", "EXCEL", "LOW", "NONE", "PREMIUM"]
 
 OCR_TYPE_VALUES: set[OcrType] = {
     "AUTO",
     "DEFAULT",
+    "EXCEL",
     "LOW",
     "NONE",
     "PREMIUM",
