@@ -1,14 +1,21 @@
+
 from __future__ import annotations
 
 import asyncio
 import time
 from http import HTTPStatus
+from io import BytesIO
 from types import SimpleNamespace
 from typing import Any, Optional, Union
 
 from docudevs.api.cases.create_case import asyncio_detailed as create_case_async
+from docudevs.api.cases.delete_case import asyncio_detailed as delete_case_async
+from docudevs.api.cases.delete_case_document import asyncio_detailed as delete_case_document_async
 from docudevs.api.cases.get_case import asyncio_detailed as get_case_async
+from docudevs.api.cases.get_case_document import asyncio_detailed as get_case_document_async
+from docudevs.api.cases.list_case_documents import asyncio_detailed as list_case_documents_async
 from docudevs.api.cases.list_cases import asyncio_detailed as list_cases_async
+from docudevs.api.cases.update_case import asyncio_detailed as update_case_async
 from docudevs.api.cases.upload_case_document import asyncio_detailed as upload_case_document_async
 from docudevs.api.batch.create_batch import asyncio_detailed as create_batch_async
 from docudevs.api.batch.process_batch import asyncio_detailed as process_batch_async
@@ -30,6 +37,7 @@ from docudevs.api.document.upload_document import (
     asyncio_detailed as upload_document_async,
 )
 from docudevs.api.document.upload_files import asyncio_detailed as upload_files_async
+from docudevs.api.document.upload_template import asyncio_detailed as upload_template_async
 from docudevs.api.job import result, status
 from docudevs.api.operations.create_generative_task import (
     asyncio as create_generative_task_async,
@@ -51,6 +59,7 @@ from docudevs.models.extraction_mode import check_extraction_mode
 from docudevs.models.generative_task_request import GenerativeTaskRequest
 from docudevs.models.template_fill_request import TemplateFillRequest
 from docudevs.models.upload_files_body import UploadFilesBody
+from docudevs.models.upload_template_body import UploadTemplateBody
 from docudevs.types import File, UNSET, Unset
 
 
@@ -72,6 +81,22 @@ class DocuDevsClient:
             raise ValueError("header.page_indices must contain only integers") from exc
         return normalized
 
+    def _wrap_response(self, response):
+        parsed = None
+        content_type = response.headers.get("content-type", "") if hasattr(response, "headers") else ""
+        has_json = isinstance(content_type, str) and "application/json" in content_type.lower()
+        if getattr(response, "content", None) and has_json:
+            try:
+                parsed = response.json()
+            except Exception:  # pragma: no cover - fallback if payload not json
+                parsed = None
+        return SimpleNamespace(
+            status_code=getattr(response, "status_code", None),
+            content=getattr(response, "content", None),
+            headers=getattr(response, "headers", None),
+            parsed=parsed,
+        )
+
     async def list_configurations(self):
         """List all named configurations."""
         return await list_configurations.asyncio_detailed(client=self._client)
@@ -220,6 +245,23 @@ class DocuDevsClient:
         """List document templates."""
         return await list_templates.asyncio_detailed(client=self._client)
 
+    async def upload_template(self, name: str, document, *, file_name: str | None = None, mime_type: str | None = None):
+        """Upload a template document."""
+        if isinstance(document, File):
+            upload_file = document
+        else:
+            if isinstance(document, (bytes, bytearray)):
+                payload = BytesIO(document)
+            else:
+                payload = document
+            if hasattr(payload, "seek"):
+                payload.seek(0)
+            upload_file = File(payload=payload, file_name=file_name, mime_type=mime_type)
+        if not upload_file.file_name:
+            upload_file.file_name = file_name or name
+        body = UploadTemplateBody(document=upload_file)
+        return await upload_template_async(client=self._client, name=name, body=body)
+
     async def metadata(self, template_id: str):
         """Get metadata for a template."""
         return await metadata.asyncio_detailed(client=self._client, name=template_id)
@@ -254,6 +296,8 @@ class DocuDevsClient:
         header_options: Optional[dict[str, Any]] = None,
         header_schema: Optional[str] = None,
         header_prompt: Optional[str] = None,
+        stop_when_empty: bool = False,
+        empty_chunk_grace: Optional[int] = None,
     ) -> UploadCommand:
         """Build an UploadCommand with map-reduce (chunked) parameters.
 
@@ -262,6 +306,8 @@ class DocuDevsClient:
         - overlap_pages >= 0
         - overlap_pages < pages_per_chunk
         - if overlap_pages > 0 then dedup_key must be provided and non-empty
+        - if stop_when_empty is true, empty_chunk_grace defaults to 0 and must be >= 0
+        - if stop_when_empty is false, empty_chunk_grace may only be None or 0
         """
         if pages_per_chunk < 1:
             raise ValueError("pages_per_chunk must be >= 1")
@@ -272,6 +318,15 @@ class DocuDevsClient:
         if overlap_pages > 0 and not dedup_key:
             raise ValueError("dedup_key is required when overlap_pages > 0")
 
+        if empty_chunk_grace is not None and empty_chunk_grace < 0:
+            raise ValueError("empty_chunk_grace must be >= 0")
+        if stop_when_empty:
+            grace_to_apply = empty_chunk_grace if empty_chunk_grace is not None else 0
+        else:
+            if empty_chunk_grace not in (None, 0):
+                raise ValueError("empty_chunk_grace requires stop_when_empty")
+            grace_to_apply = None
+
         header_model: Union[MapReduceHeaderCommand, Unset]
         header_model = UNSET
         if header_options is not None or header_schema is not None or header_prompt is not None:
@@ -339,6 +394,8 @@ class DocuDevsClient:
             pages_per_chunk=pages_per_chunk,
             overlap_pages=overlap_pages,
             dedup_key=dedup_key if dedup_key else UNSET,
+            stop_when_empty=True if stop_when_empty else UNSET,
+            empty_chunk_grace=grace_to_apply if grace_to_apply is not None else UNSET,
         )
         if not isinstance(header_model, Unset):
             mr_model.header = header_model
@@ -379,6 +436,8 @@ class DocuDevsClient:
         header_options: Optional[dict[str, Any]] = None,
         header_schema: Optional[str] = None,
         header_prompt: Optional[str] = None,
+        stop_when_empty: bool = False,
+        empty_chunk_grace: Optional[int] = None,
     ):
         """Process an already-uploaded document using map-reduce chunking parameters."""
 
@@ -438,6 +497,8 @@ class DocuDevsClient:
             header_options=header_options,
             header_schema=header_schema,
             header_prompt=header_prompt,
+            stop_when_empty=stop_when_empty,
+            empty_chunk_grace=empty_chunk_grace,
         )
 
         # Ensure JSON body includes mapReduce block even if model ignored attribute
@@ -470,6 +531,11 @@ class DocuDevsClient:
                     header_dict = _fallback_header_payload()
                     if header_dict:
                         base_map_reduce["header"] = header_dict
+                    if stop_when_empty:
+                        base_map_reduce["stopWhenEmpty"] = True
+                        base_map_reduce["emptyChunkGrace"] = (
+                            empty_chunk_grace if empty_chunk_grace is not None else 0
+                        )
                     setattr(cmd, "mapReduce", base_map_reduce)
             except Exception:
                 header_dict = _fallback_header_payload()
@@ -481,6 +547,11 @@ class DocuDevsClient:
                 }
                 if header_dict:
                     base_map_reduce["header"] = header_dict
+                if stop_when_empty:
+                    base_map_reduce["stopWhenEmpty"] = True
+                    base_map_reduce["emptyChunkGrace"] = (
+                        empty_chunk_grace if empty_chunk_grace is not None else 0
+                    )
                 setattr(cmd, "mapReduce", base_map_reduce)
         return await process_document_async(client=self._client, guid=guid, body=cmd)
 
@@ -502,6 +573,8 @@ class DocuDevsClient:
         header_options: Optional[dict[str, Any]] = None,
         header_schema: Optional[str] = None,
         header_prompt: Optional[str] = None,
+        stop_when_empty: bool = False,
+        empty_chunk_grace: Optional[int] = None,
     ) -> str:
         """Upload a document then process it using map-reduce chunking.
 
@@ -539,6 +612,8 @@ class DocuDevsClient:
             header_options=header_options,
             header_schema=header_schema,
             header_prompt=header_prompt,
+            stop_when_empty=stop_when_empty,
+            empty_chunk_grace=empty_chunk_grace,
         )
         if process_resp.status_code != HTTPStatus.OK:
             content_str = process_resp.content.decode("utf-8", errors="replace")
@@ -845,6 +920,81 @@ class DocuDevsClient:
             raise ValueError("case_id must be convertible to int")
         return await get_case_async(client=self._client, case_id=cid)
 
+    async def update_case(self, case_id: Union[int, str], body):
+        """Update an existing case."""
+        try:
+            cid = int(case_id)
+        except (TypeError, ValueError):  # pragma: no cover
+            raise ValueError("case_id must be convertible to int")
+        return await update_case_async(client=self._client, case_id=cid, body=body)
+
+    async def delete_case(self, case_id: Union[int, str]):
+        """Delete a case."""
+        try:
+            cid = int(case_id)
+        except (TypeError, ValueError):  # pragma: no cover
+            raise ValueError("case_id must be convertible to int")
+        return await delete_case_async(client=self._client, case_id=cid)
+
+    async def list_knowledge_bases(self):
+        """List cases marked as knowledge bases."""
+        response = await self._client._async_client.get("/knowledge-bases")  # type: ignore[attr-defined]
+        return self._wrap_response(response)
+
+    async def promote_knowledge_base(self, case_id: Union[int, str]):
+        """Mark a case as a knowledge base."""
+        try:
+            cid = int(case_id)
+        except (TypeError, ValueError):  # pragma: no cover
+            raise ValueError("case_id must be convertible to int")
+        response = await self._client._async_client.post(
+            "/knowledge-bases",
+            json={"caseId": cid},
+        )  # type: ignore[attr-defined]
+        return self._wrap_response(response)
+
+    async def get_knowledge_base(self, case_id: Union[int, str]):
+        """Retrieve a knowledge base case by id."""
+        try:
+            cid = int(case_id)
+        except (TypeError, ValueError):  # pragma: no cover
+            raise ValueError("case_id must be convertible to int")
+        response = await self._client._async_client.get(f"/knowledge-bases/{cid}")  # type: ignore[attr-defined]
+        return self._wrap_response(response)
+
+    async def delete_knowledge_base(self, case_id: Union[int, str]):
+        """Remove the knowledge base designation from a case."""
+        try:
+            cid = int(case_id)
+        except (TypeError, ValueError):  # pragma: no cover
+            raise ValueError("case_id must be convertible to int")
+        response = await self._client._async_client.delete(f"/knowledge-bases/{cid}")  # type: ignore[attr-defined]
+        return self._wrap_response(response)
+
+    async def list_case_documents(self, case_id: Union[int, str], *, page: int = 0, size: int = 20):
+        """List documents within a case."""
+        try:
+            cid = int(case_id)
+        except (TypeError, ValueError):  # pragma: no cover
+            raise ValueError("case_id must be convertible to int")
+        return await list_case_documents_async(client=self._client, case_id=cid, page=page, size=size)
+
+    async def get_case_document(self, case_id: Union[int, str], document_id: str):
+        """Get details for a document in a case."""
+        try:
+            cid = int(case_id)
+        except (TypeError, ValueError):  # pragma: no cover
+            raise ValueError("case_id must be convertible to int")
+        return await get_case_document_async(client=self._client, case_id=cid, document_id=document_id)
+
+    async def delete_case_document(self, case_id: Union[int, str], document_id: str):
+        """Delete a document from a case."""
+        try:
+            cid = int(case_id)
+        except (TypeError, ValueError):  # pragma: no cover
+            raise ValueError("case_id must be convertible to int")
+        return await delete_case_document_async(client=self._client, case_id=cid, document_id=document_id)
+
     async def upload_case_document(self, case_id: Union[int, str], body):
         """Upload a document to a case."""
         try:
